"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var generateCode_exports = {};
__export(generateCode_exports, {
  createImportStatements: () => createImportStatements,
  generateCode: () => generateCode,
  generateIndexCode: () => generateIndexCode
});
module.exports = __toCommonJS(generateCode_exports);
var import_path = __toESM(require("path"));
var import_utils = require("@modern-js/utils");
var templates = __toESM(require("./templates"));
var import_getClientRoutes = require("./getClientRoutes");
var import_constants = require("./constants");
var import_utils2 = require("./utils");
var import_nestedRoutes = require("./nestedRoutes");
const createImportSpecifier = (specifiers) => {
  let defaults = "";
  const named = [];
  for (const { local, imported } of specifiers) {
    if (local && imported) {
      named.push(`${imported} as ${local}`);
    } else if (local) {
      defaults = local;
    } else {
      named.push(imported);
    }
  }
  if (defaults && named.length) {
    return `${defaults}, { ${named.join(", ")} }`;
  } else if (defaults) {
    return defaults;
  } else {
    return `{ ${named.join(", ")} }`;
  }
};
const createImportStatements = (statements) => {
  const deDuplicated = [];
  const seen = /* @__PURE__ */ new Map();
  for (const { value, specifiers, initialize } of statements) {
    if (!seen.has(value)) {
      deDuplicated.push({
        value,
        specifiers,
        initialize
      });
      seen.set(value, specifiers);
    } else {
      var _deDuplicated_modifyIndex;
      seen.get(value).push(...specifiers);
      const modifyIndex = deDuplicated.findIndex((v) => v.value === value);
      var _deDuplicated_modifyIndex_initialize;
      const originInitialize = (_deDuplicated_modifyIndex_initialize = (_deDuplicated_modifyIndex = deDuplicated[modifyIndex]) === null || _deDuplicated_modifyIndex === void 0 ? void 0 : _deDuplicated_modifyIndex.initialize) !== null && _deDuplicated_modifyIndex_initialize !== void 0 ? _deDuplicated_modifyIndex_initialize : "";
      deDuplicated[modifyIndex].initialize = originInitialize.concat(`
${initialize || ""}`);
    }
  }
  return deDuplicated.map(({ value, specifiers, initialize }) => `import ${createImportSpecifier(specifiers)} from '${value}';
${initialize || ""}`).join("\n");
};
const generateCode = async (appContext, config, entrypoints, api) => {
  const { internalDirectory, srcDirectory, appDirectory, internalDirAlias, internalSrcAlias, packageName } = appContext;
  const hookRunners = api.useHookRunners();
  const isV5 = (0, import_utils.isRouterV5)(config);
  const getRoutes = isV5 ? import_getClientRoutes.getClientRoutesLegacy : import_getClientRoutes.getClientRoutes;
  const importsStatemets = /* @__PURE__ */ new Map();
  const oldVersion = typeof (config === null || config === void 0 ? void 0 : config.runtime.router) === "object" ? Boolean((config === null || config === void 0 ? void 0 : config.runtime.router).oldVersion) : false;
  await Promise.all(entrypoints.map(generateEntryCode));
  return {
    importsStatemets
  };
  async function generateEntryCode(entrypoint) {
    const { entryName, isMainEntry, isAutoMount, fileSystemRoutes } = entrypoint;
    if (isAutoMount) {
      if (fileSystemRoutes) {
        var _config_output;
        let initialRoutes = [];
        let nestedRoutes = null;
        if (entrypoint.entry) {
          initialRoutes = getRoutes({
            entrypoint,
            srcDirectory,
            srcAlias: internalSrcAlias,
            internalDirectory,
            internalDirAlias
          });
        }
        if (!isV5 && entrypoint.nestedRoutesEntry) {
          nestedRoutes = await (0, import_nestedRoutes.walk)(entrypoint.nestedRoutesEntry, entrypoint.nestedRoutesEntry, {
            name: internalSrcAlias,
            basename: srcDirectory
          }, entrypoint.entryName, entrypoint.isMainEntry, oldVersion);
          if (nestedRoutes) {
            if (!Array.isArray(nestedRoutes)) {
              nestedRoutes = [
                nestedRoutes
              ];
            }
            for (const route of nestedRoutes) {
              initialRoutes.unshift(route);
            }
          }
        }
        const { routes } = await hookRunners.modifyFileSystemRoutes({
          entrypoint,
          routes: initialRoutes
        });
        const config2 = api.useResolvedConfigContext();
        const ssr = (0, import_utils.getEntryOptions)(entryName, isMainEntry, config2.server.ssr, config2.server.ssrByEntries, packageName);
        const useSSG = (0, import_utils.isSSGEntry)(config2, entryName, entrypoints);
        let mode;
        if (ssr) {
          mode = typeof ssr === "object" ? ssr.mode || "string" : "string";
        }
        if (mode === "stream") {
          const hasPageRoute = routes.some((route) => "type" in route && route.type === "page");
          if (hasPageRoute) {
            import_utils.logger.error("Streaming ssr is not supported when pages dir exists");
            process.exit(1);
          }
        }
        const { code } = await hookRunners.beforeGenerateRoutes({
          entrypoint,
          code: await templates.fileSystemRoutes({
            routes,
            ssrMode: useSSG ? "string" : mode,
            nestedRoutesEntry: entrypoint.nestedRoutesEntry,
            entryName: entrypoint.entryName,
            internalDirectory,
            splitRouteChunks: config2 === null || config2 === void 0 ? void 0 : (_config_output = config2.output) === null || _config_output === void 0 ? void 0 : _config_output.splitRouteChunks
          })
        });
        if (entrypoint.nestedRoutesEntry && (0, import_utils.isUseSSRBundle)(config2)) {
          const routesServerFile = (0, import_utils2.getServerLoadersFile)(internalDirectory, entryName);
          const code2 = templates.routesForServer({
            routes
          });
          await import_utils.fs.ensureFile(routesServerFile);
          await import_utils.fs.writeFile(routesServerFile, code2);
        }
        const serverLoaderCombined = templates.ssrLoaderCombinedModule(entrypoints, entrypoint, config2, appContext);
        if (serverLoaderCombined) {
          const serverLoaderFile = (0, import_utils2.getServerCombinedModueFile)(internalDirectory, entryName);
          await import_utils.fs.outputFile(serverLoaderFile, serverLoaderCombined);
        }
        import_utils.fs.outputFileSync(import_path.default.resolve(internalDirectory, `./${entryName}/${import_constants.FILE_SYSTEM_ROUTES_FILE_NAME}`), code, "utf8");
      }
      const { imports } = await hookRunners.modifyEntryImports({
        entrypoint,
        imports: (0, import_utils2.getDefaultImports)({
          entrypoint,
          srcDirectory,
          appDirectory,
          internalSrcAlias,
          internalDirAlias
        })
      });
      importsStatemets.set(entryName, imports);
      const entryFile = import_path.default.resolve(internalDirectory, `./${entryName}/${import_constants.ENTRY_POINT_FILE_NAME}`);
      entrypoint.internalEntry = entryFile;
    }
  }
};
const generateIndexCode = async ({ appContext, api, entrypoints, config, importsStatemets, bundlerConfigs }) => {
  const hookRunners = api.useHookRunners();
  const { mountId } = config.html;
  const { internalDirectory, packageName } = appContext;
  await Promise.all(entrypoints.map(async (entrypoint) => {
    const { entryName, isMainEntry, isAutoMount, customBootstrap, fileSystemRoutes } = entrypoint;
    if (isAutoMount) {
      const { plugins } = await hookRunners.modifyEntryRuntimePlugins({
        entrypoint,
        plugins: [],
        bundlerConfigs
      });
      const { code: renderFunction } = await hookRunners.modifyEntryRenderFunction({
        entrypoint,
        code: templates.renderFunction({
          plugins,
          customBootstrap,
          fileSystemRoutes
        })
      });
      const { exportStatement } = await hookRunners.modifyEntryExport({
        entrypoint,
        exportStatement: "export default AppWrapper;"
      });
      const imports = importsStatemets.get(entryName);
      const code = templates.index({
        mountId,
        imports: createImportStatements(imports),
        renderFunction,
        exportStatement
      });
      const entryFile = import_path.default.resolve(internalDirectory, `./${entryName}/${import_constants.ENTRY_POINT_FILE_NAME}`);
      if (config.source.enableAsyncEntry) {
        let rawAsyncEntryCode = `import('./${import_constants.ENTRY_BOOTSTRAP_FILE_NAME}');`;
        const ssr = (0, import_utils.getEntryOptions)(entryName, isMainEntry, config.server.ssr, config.server.ssrByEntries, packageName);
        if (ssr) {
          rawAsyncEntryCode = `
        export const ${import_utils.SERVER_RENDER_FUNCTION_NAME} = async (...args) => {
          let entry = await ${rawAsyncEntryCode};
          if (entry.default instanceof Promise){
            entry = await entry.default;
            return entry.default.${import_utils.SERVER_RENDER_FUNCTION_NAME}.apply(null, args);
          }
          return entry.${import_utils.SERVER_RENDER_FUNCTION_NAME}.apply(null, args);
        };
        if(typeof window!=='undefined'){
          ${rawAsyncEntryCode}
        }
        `;
        }
        const { code: asyncEntryCode } = await hookRunners.modifyAsyncEntry({
          entrypoint,
          code: rawAsyncEntryCode
        });
        import_utils.fs.outputFileSync(entryFile, asyncEntryCode, "utf8");
        const bootstrapFile = import_path.default.resolve(internalDirectory, `./${entryName}/${import_constants.ENTRY_BOOTSTRAP_FILE_NAME}`);
        import_utils.fs.outputFileSync(bootstrapFile, code, "utf8");
      } else {
        import_utils.fs.outputFileSync(entryFile, code, "utf8");
      }
    }
  }));
};
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createImportStatements,
  generateCode,
  generateIndexCode
});
