"use strict";
var __create = Object.create;
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __getProtoOf = Object.getPrototypeOf;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toESM = (mod, isNodeMode, target) => (target = mod != null ? __create(__getProtoOf(mod)) : {}, __copyProps(
  // If the importer is in node compatibility mode or this is not an ESM
  // file that has been converted to a CommonJS file using a Babel-
  // compatible transform (i.e. "__esModule" has not been set), then set
  // "default" to the CommonJS "module.exports" for node compatibility.
  isNodeMode || !mod || !mod.__esModule ? __defProp(target, "default", { value: mod, enumerable: true }) : target,
  mod
));
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var templates_exports = {};
__export(templates_exports, {
  fileSystemRoutes: () => fileSystemRoutes,
  html: () => html,
  index: () => index,
  renderFunction: () => renderFunction,
  routesForServer: () => routesForServer,
  ssrLoaderCombinedModule: () => ssrLoaderCombinedModule
});
module.exports = __toCommonJS(templates_exports);
var import_path = __toESM(require("path"));
var import_utils = require("@modern-js/utils");
var import_constants = require("@modern-js/utils/universal/constants");
var import_constants2 = require("./constants");
var import_utils2 = require("./utils");
const index = ({ mountId, imports, renderFunction: renderFunction2, exportStatement }) => `
const IS_BROWSER = typeof window !== 'undefined' && window.name !== 'nodejs';
const IS_REACT18 = process.env.IS_REACT18 === 'true';
const MOUNT_ID = '${mountId}';

${imports}

let AppWrapper = null;

let root = null;

function render() {
  ${renderFunction2}
}

AppWrapper = render();

${exportStatement};
`;
const renderFunction = ({ plugins, customBootstrap, fileSystemRoutes: fileSystemRoutes2 }) => {
  const bootstrap = "bootstrap(AppWrapper, MOUNT_ID, root, ReactDOM)";
  return `
  const finalAppConfig = {
    ...App.config,
    ...typeof ${import_constants2.APP_CONFIG_NAME} === 'function' ? ${import_constants2.APP_CONFIG_NAME}() : {},
  }

  AppWrapper = createApp({
    plugins: [
     ${plugins.map(({ name, options, args }) => `${name}({...${options}, ...finalAppConfig?.${args || name}}),`).join("\n")}
    ]
  })(${fileSystemRoutes2 ? "" : `App`})


  if(!AppWrapper.init && typeof appInit !== 'undefined') {
    AppWrapper.init = appInit;
  }


  if (IS_BROWSER) {
    ${customBootstrap ? `customBootstrap(AppWrapper, () => ${bootstrap});` : `${bootstrap};`}
  }

  return AppWrapper
`;
};
const html = (partials) => `
<!DOCTYPE html>
<html>
<head>

  ${partials.top.join("\n")}

  ${partials.head.join("\n")}

</head>

<body>
  <div id="<%= mountId %>"><!--<?- html ?>--></div>
  ${partials.body.join("\n")}
  <!--<?- chunksMap.js ?>-->
  <!--<?- SSRDataScript ?>-->
  <!--<?- bottomTemplate ?>-->
</body>

</html>
`;
const routesForServer = ({ routes }) => {
  const loaders = [];
  const actions = [];
  const loadersMap = {};
  const traverseRouteTree = (route) => {
    let children;
    if ("children" in route && route.children) {
      var _route_children;
      children = route === null || route === void 0 ? void 0 : (_route_children = route.children) === null || _route_children === void 0 ? void 0 : _route_children.map(traverseRouteTree);
    }
    let loader;
    let action;
    if (route.type === "nested") {
      if (route.loader || route.data) {
        loaders.push(route.loader);
        const loaderId = loaders.length - 1;
        loader = `loader_${loaderId}`;
        const inline = Boolean(route.data);
        loadersMap[loader] = {
          loaderId,
          routeId: route.id,
          filePath: route.data || route.loader,
          clientData: Boolean(route.clientData),
          route,
          inline
        };
        if (route.action) {
          actions.push(route.action);
          action = `action_${loaders.length - 1}`;
        }
      }
    }
    const finalRoute = {
      ...route,
      loader,
      action,
      children
    };
    return finalRoute;
  };
  let routesCode = `
  export const routes = [
  `;
  for (const route of routes) {
    if ("type" in route) {
      const keywords = [
        "loader",
        "action"
      ];
      const regs = keywords.map(createMatchReg);
      const newRoute = traverseRouteTree(route);
      const routeStr = JSON.stringify(newRoute, null, 2);
      routesCode += regs.reduce((acc, reg) => acc.replace(reg, "$1$2"), routeStr).replace(/\\"/g, '"');
    } else {
      routesCode += `${JSON.stringify(route, null, 2)}`;
    }
  }
  routesCode += `
];`;
  let importLoadersCode = "";
  for (const [key, loaderInfo] of Object.entries(loadersMap)) {
    if (loaderInfo.inline) {
      const { route } = loaderInfo;
      if (route.action) {
        importLoadersCode += `import { loader as ${key}, action as action_${loaderInfo.loaderId} } from "${(0, import_utils.slash)(loaderInfo.filePath)}";
`;
      } else {
        importLoadersCode += `import { loader as ${key} } from "${(0, import_utils.slash)(loaderInfo.filePath)}";
`;
      }
    } else {
      importLoadersCode += `import ${key} from "${(0, import_utils.slash)(loaderInfo.filePath)}";
`;
    }
  }
  return `
    ${importLoadersCode}
    ${routesCode}
  `;
};
const createMatchReg = (keyword) => new RegExp(`("${keyword}":\\s)"([^
]+)"`, "g");
const fileSystemRoutes = async ({ routes, ssrMode, nestedRoutesEntry, entryName, internalDirectory, splitRouteChunks = true }) => {
  const components = [];
  const loadings = [];
  const errors = [];
  const loaders = [];
  const loadersMap = {};
  const configs = [];
  const configsMap = {};
  const loadersMapFile = import_path.default.join(internalDirectory, entryName, import_constants2.TEMP_LOADERS_DIR, "map.json");
  const importLazyCode = `
    import { lazy } from "react";
    import loadable, { lazy as loadableLazy } from "@modern-js/runtime/loadable"
  `;
  let rootLayoutCode = ``;
  const getDataLoaderPath = ({ loaderId, clientData, action, inline, routeId }) => {
    if (!ssrMode) {
      return "";
    }
    const clientDataStr = clientData ? `&clientData=${clientData}` : "";
    if (nestedRoutesEntry) {
      return `?loaderId=${loaderId}${clientDataStr}&action=${action ? (0, import_utils.slash)(action) : action}&inline=${inline}&routeId=${routeId}`;
    }
    return "";
  };
  const traverseRouteTree = (route) => {
    let children;
    if ("children" in route && route.children) {
      var _route_children;
      children = route === null || route === void 0 ? void 0 : (_route_children = route.children) === null || _route_children === void 0 ? void 0 : _route_children.map(traverseRouteTree);
    }
    let loading;
    let error;
    let loader;
    let action;
    let config;
    let component = "";
    let lazyImport = null;
    if (route.type === "nested") {
      if (route.loading) {
        loadings.push(route.loading);
        loading = `loading_${loadings.length - 1}`;
      }
      if (route.error) {
        errors.push(route.error);
        error = `error_${errors.length - 1}`;
      }
      if (route.loader || route.data) {
        loaders.push(route.loader);
        const loaderId = loaders.length - 1;
        loader = `loader_${loaderId}`;
        const inline = Boolean(route.data);
        loadersMap[loader] = {
          loaderId,
          routeId: route.id,
          filePath: route.data || route.loader,
          clientData: Boolean(route.clientData),
          route,
          inline
        };
        loader = `loader_${loaderId}`;
        if (route.action) {
          action = `action_${loaderId}`;
        }
      }
      if (typeof route.config === "string") {
        configs.push(route.config);
        const configId = configs.length - 1;
        config = `config_${configId}`;
        configsMap[config] = route.config;
      }
      if (route._component) {
        if (splitRouteChunks) {
          if (route.isRoot) {
            lazyImport = `() => import('${route._component}').then(routeModule => handleRouteModule(routeModule, "${route.id}")).catch(handleRouteModuleError) `;
            rootLayoutCode = `import RootLayout from '${route._component}'`;
            component = `RootLayout`;
          } else if (ssrMode === "string") {
            lazyImport = `() => import(/* webpackChunkName: "${route.id}" */  '${route._component}').then(routeModule => handleRouteModule(routeModule, "${route.id}")).catch(handleRouteModuleError) `;
            component = `loadable(${lazyImport})`;
          } else {
            lazyImport = `() => import(/* webpackChunkName: "${route.id}" */  '${route._component}').then(routeModule => handleRouteModule(routeModule, "${route.id}")).catch(handleRouteModuleError) `;
            component = `lazy(${lazyImport})`;
          }
        } else if (ssrMode === "string") {
          components.push(route._component);
          component = `component_${components.length - 1}`;
        } else {
          lazyImport = `() => import(/* webpackMode: "eager" */  '${route._component}').then(routeModule => handleRouteModule(routeModule, "${route.id}")).catch(handleRouteModuleError) `;
          component = `lazy(${lazyImport})`;
        }
      }
    } else if (route._component) {
      if (splitRouteChunks) {
        lazyImport = `() => import('${route._component}')`;
        component = `loadable(${lazyImport})`;
      } else {
        components.push(route._component);
        component = `component_${components.length - 1}`;
      }
    }
    const finalRoute = {
      ...route,
      lazyImport,
      loading,
      loader,
      action,
      config,
      error,
      children
    };
    if (route._component) {
      finalRoute.component = component;
    }
    if (route.type === "nested" && route._component && (route.loader || route.data)) {
      finalRoute.shouldRevalidate = `createShouldRevalidate("${route.id}")`;
    }
    return finalRoute;
  };
  let routeComponentsCode = `
    export const routes = [
  `;
  for (const route of routes) {
    if ("type" in route) {
      const newRoute = traverseRouteTree(route);
      const routeStr = JSON.stringify(newRoute, null, 2);
      const keywords = [
        "component",
        "lazyImport",
        "shouldRevalidate",
        "loader",
        "action",
        "loading",
        "error",
        "config"
      ];
      const regs = keywords.map(createMatchReg);
      const newRouteStr = regs.reduce((acc, reg) => acc.replace(reg, "$1$2"), routeStr).replace(/"(RootLayout)"/g, "$1").replace(/\\"/g, '"');
      routeComponentsCode += `${newRouteStr},`;
    } else {
      const component = `loadable(() => import('${route._component}'))`;
      const finalRoute = {
        ...route,
        component
      };
      const keywords = [
        "component",
        "lazyImport"
      ];
      const routeStr = JSON.stringify(finalRoute, null, 2);
      const regs = keywords.map(createMatchReg);
      const newRouteStr = regs.reduce((acc, reg) => acc.replace(reg, "$1$2"), routeStr).replace(/\\"/g, '"');
      routeComponentsCode += `${newRouteStr},`;
    }
  }
  routeComponentsCode += `
];`;
  const importLoadingCode = loadings.map((loading, index2) => {
    return `import loading_${index2} from '${loading}';
`;
  }).join("");
  const importComponentsCode = components.map((component, index2) => {
    return `import component_${index2} from '${component}';
`;
  }).join("");
  const importErrorComponentsCode = errors.map((error, index2) => {
    return `import error_${index2} from '${error}';
`;
  }).join("");
  let importLoadersCode = "";
  for (const [key, loaderInfo] of Object.entries(loadersMap)) {
    if (loaderInfo.inline) {
      const { route } = loaderInfo;
      if (route.action) {
        importLoadersCode += `import { loader as ${key}, action as action_${loaderInfo.loaderId} } from "${(0, import_utils.slash)(loaderInfo.filePath)}${getDataLoaderPath({
          loaderId: key,
          clientData: loaderInfo.clientData,
          action: route.action,
          inline: loaderInfo.inline,
          routeId: loaderInfo.routeId
        })}";
`;
      } else {
        importLoadersCode += `import { loader as ${key} } from "${(0, import_utils.slash)(loaderInfo.filePath)}${getDataLoaderPath({
          loaderId: key,
          clientData: loaderInfo.clientData,
          action: false,
          inline: loaderInfo.inline,
          routeId: route.id
        })}";
`;
      }
    } else {
      importLoadersCode += `import ${key} from "${(0, import_utils.slash)(loaderInfo.filePath)}${getDataLoaderPath({
        loaderId: key,
        clientData: loaderInfo.clientData,
        action: false,
        inline: loaderInfo.inline,
        routeId: loaderInfo.routeId
      })}";
`;
    }
  }
  let importConfigsCode = "";
  for (const [key, configPath] of Object.entries(configsMap)) {
    importConfigsCode += `import * as ${key} from "${(0, import_utils.slash)(configPath)}";
`;
  }
  await import_utils.fs.ensureFile(loadersMapFile);
  await import_utils.fs.writeJSON(loadersMapFile, loadersMap);
  const importRuntimeRouterCode = `
    import { createShouldRevalidate, handleRouteModule,  handleRouteModuleError} from '@modern-js/runtime/router';
  `;
  const routeModulesCode = `
    if(typeof document !== 'undefined'){
      window.${import_constants.ROUTE_MODULES} = {}
    }
  `;
  return `
    ${importLazyCode}
    ${importComponentsCode}
    ${importRuntimeRouterCode}
    ${rootLayoutCode}
    ${importLoadingCode}
    ${importErrorComponentsCode}
    ${importLoadersCode}
    ${importConfigsCode}
    ${routeModulesCode}
    ${routeComponentsCode}
  `;
};
function ssrLoaderCombinedModule(entrypoints, entrypoint, config, appContext) {
  const { entryName, isMainEntry } = entrypoint;
  const { packageName, internalDirectory } = appContext;
  const ssr = (0, import_utils.getEntryOptions)(entryName, isMainEntry, config.server.ssr, config.server.ssrByEntries, packageName);
  const ssg = (0, import_utils.isSSGEntry)(config, entryName, entrypoints);
  if (entrypoint.nestedRoutesEntry && (ssr || ssg)) {
    const serverLoaderRuntime = require.resolve("@modern-js/plugin-data-loader/runtime");
    const serverLoadersFile = (0, import_utils2.getServerLoadersFile)(internalDirectory, entryName);
    const combinedModule = `export * from "${(0, import_utils.slash)(serverLoaderRuntime)}"; export * from "${(0, import_utils.slash)(serverLoadersFile)}"`;
    return combinedModule;
  }
  return null;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  fileSystemRoutes,
  html,
  index,
  renderFunction,
  routesForServer,
  ssrLoaderCombinedModule
});
