"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var node_exports = {};
__export(node_exports, {
  genNodeEntry: () => genNodeEntry
});
module.exports = __toCommonJS(node_exports);
var import_utils = require("@modern-js/utils");
const severAppContextTemplate = (serverAppContext) => {
  return `{
    sharedDirectory: ${serverAppContext.sharedDirectory},
    apiDirectory: ${serverAppContext.apiDirectory},
    lambdaDirectory: ${serverAppContext.lambdaDirectory},
  }`;
};
const getPluginsCode = (plugins) => `[${plugins.map((_, index) => `plugin_${index}()`).join(",")}]`;
const genPluginImportsCode = (plugins) => {
  return plugins.map((plugin, index) => `
      let plugin_${index} = require('${plugin}')
      plugin_${index} = plugin_${index}.default || plugin_${index}
      `).join(";\n");
};
function genNodeEntry({ config, plugins, appContext } = {}) {
  const defaultConfig = {
    server: {
      port: 8080
    },
    output: {
      path: "."
    }
  };
  return `

  const fs = require('node:fs/promises');
  const path = require('node:path');
  const { createProdServer } = require('@modern-js/prod-server');
  ${genPluginImportsCode(plugins || [])}

  if(!process.env.NODE_ENV){
    process.env.NODE_ENV = 'production';
  }

  async function main() {
    try {
      let routes = [];
      const routeFilepath = path.join(__dirname, "${import_utils.ROUTE_SPEC_FILE}");
      try {
        await fs.access(routeFilepath);
        const content = await fs.readFile(routeFilepath, "utf-8");
        const routeSpec = JSON.parse(content);
        routes = routeSpec.routes;
      } catch (error) {
        console.warn('route.json not found, continuing with empty routes.');
      }

      const prodServerOptions = {
        pwd: __dirname,
        routes,
        config: ${JSON.stringify(config || defaultConfig)},
        serverConfigFile: '${import_utils.DEFAULT_SERVER_CONFIG}',
        plugins: ${getPluginsCode(plugins || [])},
        appContext: ${appContext ? severAppContextTemplate(appContext) : "undefined"},
        disableCustomHook: true
      }

      const app = await createProdServer(prodServerOptions)

      const port = process.env.PORT || 3000;

      app.listen(port, () => {
        console.log('\\x1b[32mServer is listening on port', port, '\\x1b[0m');
      });
    } catch(error) {
      console.error(error);
      process.exit(1);
    }
  }

  main();
  `;
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  genNodeEntry
});
