import path from "path";
import { ensureAbsolutePath, fs, findExists, MAIN_ENTRY_NAME, isRouterV5 } from "@modern-js/utils";
import { getFileSystemEntry } from "./getFileSystemEntry";
import { JS_EXTENSIONS } from "./constants";
import { isSubDirOrEqual } from "./utils";
const ensureExtensions = (file) => {
  if (!path.extname(file)) {
    return findExists(JS_EXTENSIONS.map((ext) => `${file}${ext}`)) || file;
  }
  return file;
};
const isDirectory = (file) => !path.extname(file);
const ifAlreadyExists = (entrypoints, checked) => entrypoints.some((entrypoint) => {
  if (ensureExtensions(entrypoint.entry) === ensureExtensions(checked.entry)) {
    checked.entryName = entrypoint.entryName;
    return true;
  }
  if (isSubDirOrEqual(entrypoint.entry, checked.entry) || isSubDirOrEqual(checked.entry, entrypoint.entry)) {
    throw new Error(`Entry configuration conflicts
 Your configuration: ${checked.entry}.
 Default entrypoint: ${entrypoint.entry}
 Please reset source.entries or set source.disableDefaultEntries to disable the default entry rules.`);
  }
  return false;
});
const getBundleEntry = (appContext, config) => {
  const { appDirectory, packageName } = appContext;
  const { disableDefaultEntries, entries, entriesDir, mainEntryName } = config.source;
  const defaults = disableDefaultEntries ? [] : getFileSystemEntry(appContext, config);
  if (entries) {
    Object.keys(entries).forEach((name) => {
      const value = entries[name];
      const entrypoint = typeof value === "string" ? {
        entryName: name,
        isMainEntry: false,
        entry: ensureAbsolutePath(appDirectory, value),
        absoluteEntryDir: isDirectory(ensureAbsolutePath(appDirectory, value)) ? ensureAbsolutePath(appDirectory, value) : path.dirname(ensureAbsolutePath(appDirectory, value)),
        isAutoMount: true,
        fileSystemRoutes: fs.statSync(ensureAbsolutePath(appDirectory, value)).isDirectory() ? {} : void 0
      } : {
        entryName: name,
        isMainEntry: false,
        entry: ensureAbsolutePath(appDirectory, value.entry),
        absoluteEntryDir: isDirectory(ensureAbsolutePath(appDirectory, value.entry)) ? ensureAbsolutePath(appDirectory, value.entry) : path.dirname(ensureAbsolutePath(appDirectory, value.entry)),
        isAutoMount: !value.disableMount,
        customBootstrap: value.customBootstrap && ensureAbsolutePath(appDirectory, value.customBootstrap),
        fileSystemRoutes: fs.statSync(ensureAbsolutePath(appDirectory, value.entry)).isDirectory() ? {} : void 0
      };
      if (entrypoint.fileSystemRoutes && !isRouterV5(config)) {
        entrypoint.nestedRoutesEntry = entrypoint.entry;
      }
      if (!ifAlreadyExists(defaults, entrypoint)) {
        defaults.push(entrypoint);
      }
    });
  }
  if (!disableDefaultEntries) {
    const entriesDirAbs = ensureAbsolutePath(appDirectory, entriesDir || "");
    const found = defaults.find(({ entryName, entry, nestedRoutesEntry = "" }) => entryName === packageName || path.dirname(entry) === entriesDirAbs || path.dirname(nestedRoutesEntry) === entriesDirAbs);
    if (found) {
      found.entryName = mainEntryName || MAIN_ENTRY_NAME;
      found.isMainEntry = true;
    }
  }
  return defaults;
};
export {
  getBundleEntry
};
