import { createUniBuilder } from "@modern-js/uni-builder";
import { createBuilderProviderConfig } from "./createBuilderProviderConfig";
import { getBuilderTargets } from "./getBuilderTargets";
import { createBuilderOptions } from "./createBuilderOptions";
async function generateBuilder(options, bundlerType) {
  const { normalizedConfig, appContext } = options;
  const builderConfig = createBuilderProviderConfig(normalizedConfig, appContext);
  const target = getBuilderTargets(normalizedConfig);
  const builderOptions = createBuilderOptions(target, appContext);
  const builder = await createUniBuilder({
    ...builderOptions,
    bundlerType,
    config: builderConfig
  });
  await applyBuilderPlugins(builder, options);
  return builder;
}
async function applyBuilderPlugins(builder, options) {
  const { builderPluginAdapterBasic, builderPluginAdapterHtml, builderPluginAdapterSSR, builderPluginAdapterWorker } = await import("../shared/builderPlugins");
  builder.addPlugins([
    builderPluginAdapterBasic(),
    builderPluginAdapterSSR(options),
    builderPluginAdapterHtml(options),
    builderPluginAdapterWorker()
  ]);
  const { normalizedConfig } = options;
  if (!normalizedConfig.output.disableNodePolyfill) {
    const { pluginNodePolyfill } = await import("@rsbuild/plugin-node-polyfill");
    builder.addPlugins([
      pluginNodePolyfill()
    ]);
  }
}
export {
  generateBuilder
};
