import path from "path";
const builderPluginAdapterBasic = () => ({
  name: "builder-plugin-adapter-modern-basic",
  setup(api) {
    api.modifyBundlerChain((chain, { target, CHAIN_ID }) => {
      if (target === "node") {
        chain.name("server");
      } else if (target === "service-worker") {
        chain.name("service-worker");
      } else if (target === "web-worker") {
        chain.name("worker");
      } else {
        chain.name("client");
      }
      if (target === "node" || target === "service-worker") {
        applyNodeCompat(target, chain);
      }
      if (target === "web") {
        const bareServerModuleReg = /\.(server|node)\.[tj]sx?$/;
        chain.module.rule(CHAIN_ID.RULE.JS).exclude.add(bareServerModuleReg);
        chain.module.rule("bare-server-module").test(bareServerModuleReg).use("server-module-loader").loader(require.resolve("../loaders/serverModuleLoader"));
      }
      chain.resolve.modules.add("node_modules").add(path.join(api.context.rootPath, "node_modules"));
    });
  }
});
function applyNodeCompat(target, chain) {
  const nodeExts = [
    ".node.js",
    ".node.jsx",
    ".node.ts",
    ".node.tsx",
    ".server.js",
    ".server.ts",
    ".server.ts",
    ".server.tsx"
  ];
  const webWorkerExts = [
    ".worker.js",
    ".worker.jsx",
    ".worker.ts",
    ".worker.tsx"
  ];
  for (const ext of nodeExts) {
    chain.resolve.extensions.prepend(ext);
  }
  if (target === "service-worker") {
    for (const ext of webWorkerExts) {
      chain.resolve.extensions.prepend(ext);
    }
  }
}
export {
  builderPluginAdapterBasic
};
