import { ResolvedConfigContext } from "@modern-js/core";
import { logger } from "@modern-js/utils";
import { generateRoutes } from "../utils/routes";
import { buildServerConfig } from "../utils/config";
import { registerCompiler } from "../utils/register";
const build = async (api, options) => {
  var _resolvedConfig_source;
  if (options === null || options === void 0 ? void 0 : options.analyze) {
    process.env.BUNDLE_ANALYZE = "true";
  }
  let resolvedConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const hookRunners = api.useHookRunners();
  await registerCompiler(appContext.appDirectory, appContext.distDirectory, resolvedConfig === null || resolvedConfig === void 0 ? void 0 : (_resolvedConfig_source = resolvedConfig.source) === null || _resolvedConfig_source === void 0 ? void 0 : _resolvedConfig_source.alias);
  const { apiOnly } = appContext;
  if (apiOnly) {
    const { appDirectory: appDirectory2, distDirectory: distDirectory2, serverConfigFile: serverConfigFile2 } = appContext;
    await hookRunners.beforeBuild({
      // "null" bundlerConfigs
      bundlerConfigs: void 0
    });
    await buildServerConfig({
      appDirectory: appDirectory2,
      distDirectory: distDirectory2,
      configFile: serverConfigFile2
    });
    await generateRoutes(appContext);
    await hookRunners.afterBuild({
      // "null" stats
      stats: void 0
    });
    return;
  }
  resolvedConfig = {
    ...resolvedConfig,
    cliOptions: options
  };
  ResolvedConfigContext.set(resolvedConfig);
  const { distDirectory, appDirectory, serverConfigFile } = appContext;
  await buildServerConfig({
    appDirectory,
    distDirectory,
    configFile: serverConfigFile
  });
  logger.info("Starting production build...");
  if (!appContext.builder) {
    throw new Error("Expect the Builder to have been initialized, But the appContext.builder received `undefined`");
  }
  await appContext.builder.build();
};
export {
  build
};
