import { ResolvedConfigContext } from "@modern-js/core";
import { DEFAULT_DEV_HOST } from "@modern-js/utils";
import { createDevServer } from "@modern-js/server";
import { initProdMiddlewares } from "@modern-js/prod-server";
import { registerCompiler } from "../utils/register";
import { printInstructions } from "../utils/printInstructions";
import { setServer } from "../utils/createServer";
import { generateRoutes } from "../utils/routes";
import { buildServerConfig } from "../utils/config";
import { getServerInternalPlugins } from "../utils/getServerInternalPlugins";
const dev = async (api, options, devServerOptions = {}) => {
  var _normalizedConfig_source, _normalizedConfig_tools;
  if (options.analyze) {
    process.env.BUNDLE_ANALYZE = "true";
  }
  let normalizedConfig = api.useResolvedConfigContext();
  const appContext = api.useAppContext();
  const hookRunners = api.useHookRunners();
  await registerCompiler(appContext.appDirectory, appContext.distDirectory, normalizedConfig === null || normalizedConfig === void 0 ? void 0 : (_normalizedConfig_source = normalizedConfig.source) === null || _normalizedConfig_source === void 0 ? void 0 : _normalizedConfig_source.alias);
  normalizedConfig = {
    ...normalizedConfig,
    cliOptions: options
  };
  ResolvedConfigContext.set(normalizedConfig);
  const { appDirectory, distDirectory, port, apiOnly, serverConfigFile, metaName, serverRoutes } = appContext;
  await buildServerConfig({
    appDirectory,
    distDirectory,
    configFile: serverConfigFile,
    watch: true
  });
  await hookRunners.beforeDev();
  if (!appContext.builder && !apiOnly) {
    throw new Error("Expect the Builder to have been initialized, But the appContext.builder received `undefined`");
  }
  await generateRoutes(appContext);
  const serverInternalPlugins = await getServerInternalPlugins(api);
  const serverOptions = {
    metaName,
    dev: {
      port,
      https: normalizedConfig.dev.https,
      host: normalizedConfig.dev.host,
      ...(_normalizedConfig_tools = normalizedConfig.tools) === null || _normalizedConfig_tools === void 0 ? void 0 : _normalizedConfig_tools.devServer
    },
    appContext: {
      appDirectory,
      apiDirectory: appContext.apiDirectory,
      lambdaDirectory: appContext.lambdaDirectory,
      sharedDirectory: appContext.sharedDirectory
    },
    routes: serverRoutes,
    pwd: appDirectory,
    config: normalizedConfig,
    serverConfigFile,
    internalPlugins: serverInternalPlugins,
    ...devServerOptions
  };
  if (apiOnly) {
    var _normalizedConfig_dev;
    const app = await createDevServer(serverOptions, initProdMiddlewares);
    const host = ((_normalizedConfig_dev = normalizedConfig.dev) === null || _normalizedConfig_dev === void 0 ? void 0 : _normalizedConfig_dev.host) || DEFAULT_DEV_HOST;
    app.listen({
      port,
      host
    }, () => {
      printInstructions(hookRunners, appContext, normalizedConfig);
    });
  } else {
    const { server } = await appContext.builder.startDevServer({
      serverOptions,
      initProdMiddlewares
    });
    setServer(server);
  }
};
export {
  dev
};
