import { logger, isApiOnly, getTargetDir } from "@modern-js/utils";
import { createProdServer } from "@modern-js/prod-server";
import { printInstructions } from "../utils/printInstructions";
import { getServerInternalPlugins } from "../utils/getServerInternalPlugins";
const start = async (api) => {
  var _userConfig_source, _userConfig_output_distPath;
  const appContext = api.useAppContext();
  const userConfig = api.useResolvedConfigContext();
  const hookRunners = api.useHookRunners();
  const { distDirectory, appDirectory, port, serverConfigFile, metaName, serverRoutes } = appContext;
  logger.info(`Starting production server...`);
  const apiOnly = await isApiOnly(appContext.appDirectory, userConfig === null || userConfig === void 0 ? void 0 : (_userConfig_source = userConfig.source) === null || _userConfig_source === void 0 ? void 0 : _userConfig_source.entriesDir, appContext.apiDirectory);
  let runMode;
  if (apiOnly) {
    runMode = "apiOnly";
  }
  const serverInternalPlugins = await getServerInternalPlugins(api);
  const app = await createProdServer({
    metaName,
    pwd: distDirectory,
    config: {
      ...userConfig,
      dev: userConfig.dev,
      output: {
        path: (_userConfig_output_distPath = userConfig.output.distPath) === null || _userConfig_output_distPath === void 0 ? void 0 : _userConfig_output_distPath.root,
        ...userConfig.output || {}
      }
    },
    routes: serverRoutes,
    appContext: {
      appDirectory,
      sharedDirectory: getTargetDir(appContext.sharedDirectory, appContext.appDirectory, appContext.distDirectory),
      apiDirectory: getTargetDir(appContext.apiDirectory, appContext.appDirectory, appContext.distDirectory),
      lambdaDirectory: getTargetDir(appContext.lambdaDirectory, appContext.appDirectory, appContext.distDirectory)
    },
    serverConfigFile,
    internalPlugins: serverInternalPlugins,
    runMode
  });
  app.listen(port, async () => {
    await printInstructions(hookRunners, appContext, userConfig);
  });
};
export {
  start
};
