import { ROUTE_SPEC_FILE, DEFAULT_SERVER_CONFIG } from "@modern-js/utils";
const severAppContextTemplate = (serverAppContext) => {
  return `{
    sharedDirectory: ${serverAppContext.sharedDirectory},
    apiDirectory: ${serverAppContext.apiDirectory},
    lambdaDirectory: ${serverAppContext.lambdaDirectory},
  }`;
};
const getPluginsCode = (plugins) => `[${plugins.map((_, index) => `plugin_${index}()`).join(",")}]`;
const genPluginImportsCode = (plugins) => {
  return plugins.map((plugin, index) => `
      let plugin_${index} = require('${plugin}')
      plugin_${index} = plugin_${index}.default || plugin_${index}
      `).join(";\n");
};
function genNodeEntry({ config, plugins, appContext } = {}) {
  const defaultConfig = {
    server: {
      port: 8080
    },
    output: {
      path: "."
    }
  };
  return `

  const fs = require('node:fs/promises');
  const path = require('node:path');
  const { createProdServer } = require('@modern-js/prod-server');
  ${genPluginImportsCode(plugins || [])}

  if(!process.env.NODE_ENV){
    process.env.NODE_ENV = 'production';
  }

  async function main() {
    try {
      let routes = [];
      const routeFilepath = path.join(__dirname, "${ROUTE_SPEC_FILE}");
      try {
        await fs.access(routeFilepath);
        const content = await fs.readFile(routeFilepath, "utf-8");
        const routeSpec = JSON.parse(content);
        routes = routeSpec.routes;
      } catch (error) {
        console.warn('route.json not found, continuing with empty routes.');
      }

      const prodServerOptions = {
        pwd: __dirname,
        routes,
        config: ${JSON.stringify(config || defaultConfig)},
        serverConfigFile: '${DEFAULT_SERVER_CONFIG}',
        plugins: ${getPluginsCode(plugins || [])},
        appContext: ${appContext ? severAppContextTemplate(appContext) : "undefined"},
        disableCustomHook: true
      }

      const app = await createProdServer(prodServerOptions)

      const port = process.env.PORT || 3000;

      app.listen(port, () => {
        console.log('\\x1b[32mServer is listening on port', port, '\\x1b[0m');
      });
    } catch(error) {
      console.error(error);
      process.exit(1);
    }
  }

  main();
  `;
}
export {
  genNodeEntry
};
