import path from "node:path";
import { fs, getAliasConfig, readTsConfigByFile } from "@modern-js/utils";
const registerCompiler = async (appDir = process.cwd(), distDir, alias) => {
  const TS_CONFIG_FILENAME = `tsconfig.json`;
  const tsconfigPath = path.resolve(appDir, TS_CONFIG_FILENAME);
  const isTsProject = await fs.pathExists(tsconfigPath);
  const aliasConfig = getAliasConfig(alias, {
    appDirectory: appDir,
    tsconfigPath
  });
  const { paths = {}, absoluteBaseUrl = "./" } = aliasConfig;
  const tsPaths = Object.keys(paths).reduce((o, key) => {
    let tsPath = paths[key];
    if (typeof tsPath === "string" && path.isAbsolute(tsPath)) {
      tsPath = path.relative(absoluteBaseUrl, tsPath);
    }
    if (typeof tsPath === "string") {
      tsPath = [
        tsPath
      ];
    }
    return {
      ...o,
      [`${key}`]: tsPath
    };
  }, {});
  let tsConfig = {};
  if (isTsProject) {
    tsConfig = readTsConfigByFile(tsconfigPath);
  }
  try {
    const tsNode = await import("ts-node");
    const tsNodeOptions = tsConfig["ts-node"];
    if (isTsProject) {
      tsNode.register({
        project: tsconfigPath,
        scope: true,
        // for env.d.ts, https://www.npmjs.com/package/ts-node#missing-types
        files: true,
        transpileOnly: true,
        ignore: [
          "(?:^|/)node_modules/",
          `(?:^|/)${distDir}/`
        ],
        ...tsNodeOptions
      });
    }
  } catch (error) {
    const esbuildRegister = await import("esbuild-register/dist/node");
    esbuildRegister.register({
      tsconfigRaw: isTsProject ? tsConfig : void 0,
      hookIgnoreNodeModules: true,
      hookMatcher: (fileName) => !fileName.startsWith(distDir)
    });
  }
  const tsConfigPaths = await import("@modern-js/utils/tsconfig-paths");
  if (await fs.pathExists(appDir)) {
    const loaderRes = tsConfigPaths.loadConfig(appDir);
    if (loaderRes.resultType === "success") {
      tsConfigPaths.register({
        baseUrl: absoluteBaseUrl || "./",
        paths: tsPaths
      });
    }
  }
};
export {
  registerCompiler
};
