import { _ as _async_to_generator } from "@swc/helpers/_/_async_to_generator";
import { _ as _to_consumable_array } from "@swc/helpers/_/_to_consumable_array";
import { _ as _ts_generator } from "@swc/helpers/_/_ts_generator";
import path from "path";
import { fs, getEntryOptions, isRouterV5, isSSGEntry, isUseSSRBundle, logger, SERVER_RENDER_FUNCTION_NAME } from "@modern-js/utils";
import * as templates from "./templates";
import { getClientRoutes, getClientRoutesLegacy } from "./getClientRoutes";
import { FILE_SYSTEM_ROUTES_FILE_NAME, ENTRY_POINT_FILE_NAME, ENTRY_BOOTSTRAP_FILE_NAME } from "./constants";
import { getDefaultImports, getServerLoadersFile, getServerCombinedModueFile } from "./utils";
import { walk } from "./nestedRoutes";
var createImportSpecifier = function(specifiers) {
  var defaults = "";
  var named = [];
  var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
  try {
    for (var _iterator = specifiers[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _step_value = _step.value, local = _step_value.local, imported = _step_value.imported;
      if (local && imported) {
        named.push("".concat(imported, " as ").concat(local));
      } else if (local) {
        defaults = local;
      } else {
        named.push(imported);
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
  if (defaults && named.length) {
    return "".concat(defaults, ", { ").concat(named.join(", "), " }");
  } else if (defaults) {
    return defaults;
  } else {
    return "{ ".concat(named.join(", "), " }");
  }
};
var createImportStatements = function(statements) {
  var deDuplicated = [];
  var seen = /* @__PURE__ */ new Map();
  var _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
  try {
    var _loop = function() {
      var _step_value = _step.value, value = _step_value.value, specifiers = _step_value.specifiers, initialize = _step_value.initialize;
      if (!seen.has(value)) {
        deDuplicated.push({
          value,
          specifiers,
          initialize
        });
        seen.set(value, specifiers);
      } else {
        var _seen_get;
        var _deDuplicated_modifyIndex;
        (_seen_get = seen.get(value)).push.apply(_seen_get, _to_consumable_array(specifiers));
        var modifyIndex = deDuplicated.findIndex(function(v) {
          return v.value === value;
        });
        var _deDuplicated_modifyIndex_initialize;
        var originInitialize = (_deDuplicated_modifyIndex_initialize = (_deDuplicated_modifyIndex = deDuplicated[modifyIndex]) === null || _deDuplicated_modifyIndex === void 0 ? void 0 : _deDuplicated_modifyIndex.initialize) !== null && _deDuplicated_modifyIndex_initialize !== void 0 ? _deDuplicated_modifyIndex_initialize : "";
        deDuplicated[modifyIndex].initialize = originInitialize.concat("\n".concat(initialize || ""));
      }
    };
    for (var _iterator = statements[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true)
      _loop();
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return != null) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
  return deDuplicated.map(function(param) {
    var value = param.value, specifiers = param.specifiers, initialize = param.initialize;
    return "import ".concat(createImportSpecifier(specifiers), " from '").concat(value, "';\n").concat(initialize || "");
  }).join("\n");
};
var generateCode = function() {
  var _ref = _async_to_generator(function(appContext, config, entrypoints, api) {
    var internalDirectory, srcDirectory, appDirectory, internalDirAlias, internalSrcAlias, packageName, hookRunners, isV5, getRoutes, importsStatemets, oldVersion;
    function generateEntryCode(entrypoint) {
      return _generateEntryCode.apply(this, arguments);
    }
    function _generateEntryCode() {
      _generateEntryCode = _async_to_generator(function(entrypoint) {
        var entryName, isMainEntry, isAutoMount, fileSystemRoutes, _config_output, initialRoutes, nestedRoutes, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, route, routes, _$config, ssr, useSSG, mode, hasPageRoute, code, _, _tmp, routesServerFile, code1, serverLoaderCombined, serverLoaderFile, imports, entryFile;
        return _ts_generator(this, function(_state) {
          switch (_state.label) {
            case 0:
              entryName = entrypoint.entryName, isMainEntry = entrypoint.isMainEntry, isAutoMount = entrypoint.isAutoMount, fileSystemRoutes = entrypoint.fileSystemRoutes;
              if (!isAutoMount)
                return [
                  3,
                  13
                ];
              if (!fileSystemRoutes)
                return [
                  3,
                  11
                ];
              initialRoutes = [];
              nestedRoutes = null;
              if (entrypoint.entry) {
                initialRoutes = getRoutes({
                  entrypoint,
                  srcDirectory,
                  srcAlias: internalSrcAlias,
                  internalDirectory,
                  internalDirAlias
                });
              }
              if (!(!isV5 && entrypoint.nestedRoutesEntry))
                return [
                  3,
                  2
                ];
              return [
                4,
                walk(entrypoint.nestedRoutesEntry, entrypoint.nestedRoutesEntry, {
                  name: internalSrcAlias,
                  basename: srcDirectory
                }, entrypoint.entryName, entrypoint.isMainEntry, oldVersion)
              ];
            case 1:
              nestedRoutes = _state.sent();
              if (nestedRoutes) {
                if (!Array.isArray(nestedRoutes)) {
                  nestedRoutes = [
                    nestedRoutes
                  ];
                }
                _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
                try {
                  for (_iterator = nestedRoutes[Symbol.iterator](); !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    route = _step.value;
                    initialRoutes.unshift(route);
                  }
                } catch (err) {
                  _didIteratorError = true;
                  _iteratorError = err;
                } finally {
                  try {
                    if (!_iteratorNormalCompletion && _iterator.return != null) {
                      _iterator.return();
                    }
                  } finally {
                    if (_didIteratorError) {
                      throw _iteratorError;
                    }
                  }
                }
              }
              _state.label = 2;
            case 2:
              return [
                4,
                hookRunners.modifyFileSystemRoutes({
                  entrypoint,
                  routes: initialRoutes
                })
              ];
            case 3:
              routes = _state.sent().routes;
              _$config = api.useResolvedConfigContext();
              ssr = getEntryOptions(entryName, isMainEntry, _$config.server.ssr, _$config.server.ssrByEntries, packageName);
              useSSG = isSSGEntry(_$config, entryName, entrypoints);
              if (ssr) {
                mode = typeof ssr === "object" ? ssr.mode || "string" : "string";
              }
              if (mode === "stream") {
                hasPageRoute = routes.some(function(route2) {
                  return "type" in route2 && route2.type === "page";
                });
                if (hasPageRoute) {
                  logger.error("Streaming ssr is not supported when pages dir exists");
                  process.exit(1);
                }
              }
              _ = hookRunners.beforeGenerateRoutes;
              _tmp = {
                entrypoint
              };
              return [
                4,
                templates.fileSystemRoutes({
                  routes,
                  ssrMode: useSSG ? "string" : mode,
                  nestedRoutesEntry: entrypoint.nestedRoutesEntry,
                  entryName: entrypoint.entryName,
                  internalDirectory,
                  splitRouteChunks: _$config === null || _$config === void 0 ? void 0 : (_config_output = _$config.output) === null || _config_output === void 0 ? void 0 : _config_output.splitRouteChunks
                })
              ];
            case 4:
              return [
                4,
                _.apply(hookRunners, [
                  (_tmp.code = _state.sent(), _tmp)
                ])
              ];
            case 5:
              code = _state.sent().code;
              if (!(entrypoint.nestedRoutesEntry && isUseSSRBundle(_$config)))
                return [
                  3,
                  8
                ];
              routesServerFile = getServerLoadersFile(internalDirectory, entryName);
              code1 = templates.routesForServer({
                routes
              });
              return [
                4,
                fs.ensureFile(routesServerFile)
              ];
            case 6:
              _state.sent();
              return [
                4,
                fs.writeFile(routesServerFile, code1)
              ];
            case 7:
              _state.sent();
              _state.label = 8;
            case 8:
              serverLoaderCombined = templates.ssrLoaderCombinedModule(entrypoints, entrypoint, _$config, appContext);
              if (!serverLoaderCombined)
                return [
                  3,
                  10
                ];
              serverLoaderFile = getServerCombinedModueFile(internalDirectory, entryName);
              return [
                4,
                fs.outputFile(serverLoaderFile, serverLoaderCombined)
              ];
            case 9:
              _state.sent();
              _state.label = 10;
            case 10:
              fs.outputFileSync(path.resolve(internalDirectory, "./".concat(entryName, "/").concat(FILE_SYSTEM_ROUTES_FILE_NAME)), code, "utf8");
              _state.label = 11;
            case 11:
              return [
                4,
                hookRunners.modifyEntryImports({
                  entrypoint,
                  imports: getDefaultImports({
                    entrypoint,
                    srcDirectory,
                    appDirectory,
                    internalSrcAlias,
                    internalDirAlias
                  })
                })
              ];
            case 12:
              imports = _state.sent().imports;
              importsStatemets.set(entryName, imports);
              entryFile = path.resolve(internalDirectory, "./".concat(entryName, "/").concat(ENTRY_POINT_FILE_NAME));
              entrypoint.internalEntry = entryFile;
              _state.label = 13;
            case 13:
              return [
                2
              ];
          }
        });
      });
      return _generateEntryCode.apply(this, arguments);
    }
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          internalDirectory = appContext.internalDirectory, srcDirectory = appContext.srcDirectory, appDirectory = appContext.appDirectory, internalDirAlias = appContext.internalDirAlias, internalSrcAlias = appContext.internalSrcAlias, packageName = appContext.packageName;
          hookRunners = api.useHookRunners();
          isV5 = isRouterV5(config);
          getRoutes = isV5 ? getClientRoutesLegacy : getClientRoutes;
          importsStatemets = /* @__PURE__ */ new Map();
          oldVersion = typeof (config === null || config === void 0 ? void 0 : config.runtime.router) === "object" ? Boolean((config === null || config === void 0 ? void 0 : config.runtime.router).oldVersion) : false;
          return [
            4,
            Promise.all(entrypoints.map(generateEntryCode))
          ];
        case 1:
          _state.sent();
          return [
            2,
            {
              importsStatemets
            }
          ];
      }
    });
  });
  return function generateCode2(appContext, config, entrypoints, api) {
    return _ref.apply(this, arguments);
  };
}();
var generateIndexCode = function() {
  var _ref = _async_to_generator(function(param) {
    var appContext, api, entrypoints, config, importsStatemets, bundlerConfigs, hookRunners, mountId, internalDirectory, packageName;
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          appContext = param.appContext, api = param.api, entrypoints = param.entrypoints, config = param.config, importsStatemets = param.importsStatemets, bundlerConfigs = param.bundlerConfigs;
          hookRunners = api.useHookRunners();
          mountId = config.html.mountId;
          internalDirectory = appContext.internalDirectory, packageName = appContext.packageName;
          return [
            4,
            Promise.all(entrypoints.map(function() {
              var _ref2 = _async_to_generator(function(entrypoint) {
                var entryName, isMainEntry, isAutoMount, customBootstrap, fileSystemRoutes, plugins, _ref3, renderFunction, exportStatement, imports, code, entryFile, rawAsyncEntryCode, ssr, _ref1, asyncEntryCode, bootstrapFile;
                return _ts_generator(this, function(_state2) {
                  switch (_state2.label) {
                    case 0:
                      entryName = entrypoint.entryName, isMainEntry = entrypoint.isMainEntry, isAutoMount = entrypoint.isAutoMount, customBootstrap = entrypoint.customBootstrap, fileSystemRoutes = entrypoint.fileSystemRoutes;
                      if (!isAutoMount)
                        return [
                          3,
                          6
                        ];
                      return [
                        4,
                        hookRunners.modifyEntryRuntimePlugins({
                          entrypoint,
                          plugins: [],
                          bundlerConfigs
                        })
                      ];
                    case 1:
                      plugins = _state2.sent().plugins;
                      return [
                        4,
                        hookRunners.modifyEntryRenderFunction({
                          entrypoint,
                          code: templates.renderFunction({
                            plugins,
                            customBootstrap,
                            fileSystemRoutes
                          })
                        })
                      ];
                    case 2:
                      _ref3 = _state2.sent(), renderFunction = _ref3.code;
                      return [
                        4,
                        hookRunners.modifyEntryExport({
                          entrypoint,
                          exportStatement: "export default AppWrapper;"
                        })
                      ];
                    case 3:
                      exportStatement = _state2.sent().exportStatement;
                      imports = importsStatemets.get(entryName);
                      code = templates.index({
                        mountId,
                        imports: createImportStatements(imports),
                        renderFunction,
                        exportStatement
                      });
                      entryFile = path.resolve(internalDirectory, "./".concat(entryName, "/").concat(ENTRY_POINT_FILE_NAME));
                      if (!config.source.enableAsyncEntry)
                        return [
                          3,
                          5
                        ];
                      rawAsyncEntryCode = "import('./".concat(ENTRY_BOOTSTRAP_FILE_NAME, "');");
                      ssr = getEntryOptions(entryName, isMainEntry, config.server.ssr, config.server.ssrByEntries, packageName);
                      if (ssr) {
                        rawAsyncEntryCode = "\n        export const ".concat(SERVER_RENDER_FUNCTION_NAME, " = async (...args) => {\n          let entry = await ").concat(rawAsyncEntryCode, ";\n          if (entry.default instanceof Promise){\n            entry = await entry.default;\n            return entry.default.").concat(SERVER_RENDER_FUNCTION_NAME, ".apply(null, args);\n          }\n          return entry.").concat(SERVER_RENDER_FUNCTION_NAME, ".apply(null, args);\n        };\n        if(typeof window!=='undefined'){\n          ").concat(rawAsyncEntryCode, "\n        }\n        ");
                      }
                      return [
                        4,
                        hookRunners.modifyAsyncEntry({
                          entrypoint,
                          code: rawAsyncEntryCode
                        })
                      ];
                    case 4:
                      _ref1 = _state2.sent(), asyncEntryCode = _ref1.code;
                      fs.outputFileSync(entryFile, asyncEntryCode, "utf8");
                      bootstrapFile = path.resolve(internalDirectory, "./".concat(entryName, "/").concat(ENTRY_BOOTSTRAP_FILE_NAME));
                      fs.outputFileSync(bootstrapFile, code, "utf8");
                      return [
                        3,
                        6
                      ];
                    case 5:
                      fs.outputFileSync(entryFile, code, "utf8");
                      _state2.label = 6;
                    case 6:
                      return [
                        2
                      ];
                  }
                });
              });
              return function(entrypoint) {
                return _ref2.apply(this, arguments);
              };
            }()))
          ];
        case 1:
          _state.sent();
          return [
            2
          ];
      }
    });
  });
  return function generateIndexCode2(_) {
    return _ref.apply(this, arguments);
  };
}();
export {
  createImportStatements,
  generateCode,
  generateIndexCode
};
