import path from "path";
import { ensureAbsolutePath, fs, findExists, MAIN_ENTRY_NAME, isRouterV5 } from "@modern-js/utils";
import { getFileSystemEntry } from "./getFileSystemEntry";
import { JS_EXTENSIONS } from "./constants";
import { isSubDirOrEqual } from "./utils";
var ensureExtensions = function(file) {
  if (!path.extname(file)) {
    return findExists(JS_EXTENSIONS.map(function(ext) {
      return "".concat(file).concat(ext);
    })) || file;
  }
  return file;
};
var isDirectory = function(file) {
  return !path.extname(file);
};
var ifAlreadyExists = function(entrypoints, checked) {
  return entrypoints.some(function(entrypoint) {
    if (ensureExtensions(entrypoint.entry) === ensureExtensions(checked.entry)) {
      checked.entryName = entrypoint.entryName;
      return true;
    }
    if (isSubDirOrEqual(entrypoint.entry, checked.entry) || isSubDirOrEqual(checked.entry, entrypoint.entry)) {
      throw new Error("Entry configuration conflicts\n Your configuration: ".concat(checked.entry, ".\n Default entrypoint: ").concat(entrypoint.entry, "\n Please reset source.entries or set source.disableDefaultEntries to disable the default entry rules."));
    }
    return false;
  });
};
var getBundleEntry = function(appContext, config) {
  var appDirectory = appContext.appDirectory, packageName = appContext.packageName;
  var _config_source = config.source, disableDefaultEntries = _config_source.disableDefaultEntries, entries = _config_source.entries, entriesDir = _config_source.entriesDir, mainEntryName = _config_source.mainEntryName;
  var defaults = disableDefaultEntries ? [] : getFileSystemEntry(appContext, config);
  if (entries) {
    Object.keys(entries).forEach(function(name) {
      var value = entries[name];
      var entrypoint = typeof value === "string" ? {
        entryName: name,
        isMainEntry: false,
        entry: ensureAbsolutePath(appDirectory, value),
        absoluteEntryDir: isDirectory(ensureAbsolutePath(appDirectory, value)) ? ensureAbsolutePath(appDirectory, value) : path.dirname(ensureAbsolutePath(appDirectory, value)),
        isAutoMount: true,
        fileSystemRoutes: fs.statSync(ensureAbsolutePath(appDirectory, value)).isDirectory() ? {} : void 0
      } : {
        entryName: name,
        isMainEntry: false,
        entry: ensureAbsolutePath(appDirectory, value.entry),
        absoluteEntryDir: isDirectory(ensureAbsolutePath(appDirectory, value.entry)) ? ensureAbsolutePath(appDirectory, value.entry) : path.dirname(ensureAbsolutePath(appDirectory, value.entry)),
        isAutoMount: !value.disableMount,
        customBootstrap: value.customBootstrap && ensureAbsolutePath(appDirectory, value.customBootstrap),
        fileSystemRoutes: fs.statSync(ensureAbsolutePath(appDirectory, value.entry)).isDirectory() ? {} : void 0
      };
      if (entrypoint.fileSystemRoutes && !isRouterV5(config)) {
        entrypoint.nestedRoutesEntry = entrypoint.entry;
      }
      if (!ifAlreadyExists(defaults, entrypoint)) {
        defaults.push(entrypoint);
      }
    });
  }
  if (!disableDefaultEntries) {
    var entriesDirAbs = ensureAbsolutePath(appDirectory, entriesDir || "");
    var found = defaults.find(function(param) {
      var entryName = param.entryName, entry = param.entry, _param_nestedRoutesEntry = param.nestedRoutesEntry, nestedRoutesEntry = _param_nestedRoutesEntry === void 0 ? "" : _param_nestedRoutesEntry;
      return entryName === packageName || path.dirname(entry) === entriesDirAbs || path.dirname(nestedRoutesEntry) === entriesDirAbs;
    });
    if (found) {
      found.entryName = mainEntryName || MAIN_ENTRY_NAME;
      found.isMainEntry = true;
    }
  }
  return defaults;
};
export {
  getBundleEntry
};
