import fs from "fs";
import path from "path";
import { findExists, ensureAbsolutePath } from "@modern-js/utils";
import { isDefaultExportFunction } from "./isDefaultExportFunction";
import { JS_EXTENSIONS, INDEX_FILE_NAME, APP_FILE_NAME, PAGES_DIR_NAME, FILE_SYSTEM_ROUTES_GLOBAL_LAYOUT, NESTED_ROUTES_DIR } from "./constants";
var hasIndex = function(dir) {
  return findExists(JS_EXTENSIONS.map(function(ext) {
    return path.resolve(dir, "".concat(INDEX_FILE_NAME).concat(ext));
  }));
};
var hasApp = function(dir) {
  return findExists(JS_EXTENSIONS.map(function(ext) {
    return path.resolve(dir, "".concat(APP_FILE_NAME).concat(ext));
  }));
};
var hasPages = function(dir) {
  return fs.existsSync(path.join(dir, PAGES_DIR_NAME));
};
var hasNestedRoutes = function(dir) {
  return fs.existsSync(path.join(dir, NESTED_ROUTES_DIR));
};
var isBundleEntry = function(dir) {
  return hasApp(dir) || hasPages(dir) || hasIndex(dir) || hasNestedRoutes(dir);
};
var scanDir = function(dirs) {
  return dirs.map(function(dir) {
    var indexFile = hasIndex(dir);
    var customBootstrap = isDefaultExportFunction(indexFile) ? indexFile : false;
    var entryName = path.basename(dir);
    if (indexFile && !customBootstrap) {
      return {
        entryName,
        isMainEntry: false,
        entry: indexFile,
        absoluteEntryDir: path.resolve(dir),
        isAutoMount: false
      };
    }
    var isHasApp = hasApp(dir);
    if (isHasApp) {
      return {
        entryName,
        isMainEntry: false,
        entry: path.join(dir, APP_FILE_NAME),
        isAutoMount: true,
        absoluteEntryDir: path.resolve(dir),
        customBootstrap
      };
    }
    var isHasNestedRoutes = hasNestedRoutes(dir);
    var isHasPages = hasPages(dir);
    if (isHasNestedRoutes || isHasPages) {
      var entrypoint = {
        entryName,
        isMainEntry: false,
        entry: "",
        fileSystemRoutes: {
          globalApp: findExists(JS_EXTENSIONS.map(function(ext) {
            return path.resolve(dir, "./".concat(PAGES_DIR_NAME, "/").concat(FILE_SYSTEM_ROUTES_GLOBAL_LAYOUT).concat(ext));
          }))
        },
        isAutoMount: true,
        absoluteEntryDir: path.resolve(dir),
        customBootstrap
      };
      if (isHasPages) {
        entrypoint.entry = path.join(dir, PAGES_DIR_NAME);
        entrypoint.pageRoutesEntry = entrypoint.entry;
      }
      if (isHasNestedRoutes) {
        entrypoint.entry = path.join(dir, NESTED_ROUTES_DIR);
        entrypoint.nestedRoutesEntry = entrypoint.entry;
      }
      return entrypoint;
    }
    return {
      entryName,
      isMainEntry: false,
      entry: indexFile,
      absoluteEntryDir: path.resolve(dir),
      isAutoMount: false
    };
  });
};
var getFileSystemEntry = function(appContext, config) {
  var appDirectory = appContext.appDirectory;
  var _config_source = config.source, entriesDir = _config_source.entriesDir, disableEntryDirs = _config_source.disableEntryDirs;
  var disabledDirs = [];
  if (disableEntryDirs && Array.isArray(disableEntryDirs)) {
    disabledDirs = disableEntryDirs === null || disableEntryDirs === void 0 ? void 0 : disableEntryDirs.map(function(dir) {
      return ensureAbsolutePath(appDirectory, dir);
    });
  }
  var src = ensureAbsolutePath(appDirectory, entriesDir || "");
  if (fs.existsSync(src)) {
    if (fs.statSync(src).isDirectory()) {
      return scanDir(isBundleEntry(src) ? [
        src
      ] : fs.readdirSync(src).map(function(file) {
        return path.join(src, file);
      }).filter(function(file) {
        return fs.statSync(file).isDirectory() && isBundleEntry(file) && !disabledDirs.includes(file);
      }));
    } else {
      throw Error("source.entriesDir accept a directory.");
    }
  } else {
    throw Error("src dir ".concat(entriesDir, " not found."));
  }
};
export {
  getFileSystemEntry
};
