import { _ as _async_to_generator } from "@swc/helpers/_/_async_to_generator";
import { _ as _object_spread } from "@swc/helpers/_/_object_spread";
import { _ as _object_spread_props } from "@swc/helpers/_/_object_spread_props";
import { _ as _sliced_to_array } from "@swc/helpers/_/_sliced_to_array";
import { _ as _ts_generator } from "@swc/helpers/_/_ts_generator";
import * as path from "path";
import { createDebugger, findExists, fs, isApiOnly, minimist, isDevCommand, getArgv } from "@modern-js/utils";
import { cloneDeep } from "@modern-js/utils/lodash";
import { printInstructions } from "../utils/printInstructions";
import { generateRoutes, getPathWithoutExt } from "../utils/routes";
import { emitResolvedConfig } from "../utils/config";
import { getSelectedEntries } from "../utils/getSelectedEntries";
import { initialNormalizedConfig } from "../config";
import { createBuilderGenerator } from "../builder";
import { checkIsBuildCommands, isPageComponentFile, parseModule, replaceWithAlias } from "./utils";
import { APP_CONFIG_NAME, APP_INIT_EXPORTED, APP_INIT_IMPORTED } from "./constants";
import { generateIndexCode } from "./generateCode";
var debug = createDebugger("plugin-analyze");
function analyze_default(param) {
  var bundler = param.bundler;
  return {
    name: "@modern-js/plugin-analyze",
    setup: function(api) {
      var pagesDir = [];
      var nestedRouteEntries = [];
      var originEntrypoints = [];
      return {
        prepare: function prepare() {
          return _async_to_generator(function() {
            var _resolvedConfig_source, appContext, resolvedConfig, hookRunners, apiOnly, routes, _ref, getBundleEntry, getServerRoutes, generateCode, getHtmlTemplate, entrypoints, initialRoutes, routes1, importsStatemets, htmlTemplates, checkedEntries, entry, normalizedConfig, createBuilderForModern, builder;
            return _ts_generator(this, function(_state) {
              switch (_state.label) {
                case 0:
                  appContext = api.useAppContext();
                  resolvedConfig = api.useResolvedConfigContext();
                  hookRunners = api.useHookRunners();
                  try {
                    if (checkIsBuildCommands()) {
                      fs.emptydirSync(appContext.internalDirectory);
                    }
                  } catch (e) {
                  }
                  return [
                    4,
                    isApiOnly(appContext.appDirectory, (_resolvedConfig_source = resolvedConfig.source) === null || _resolvedConfig_source === void 0 ? void 0 : _resolvedConfig_source.entriesDir, appContext.apiDirectory)
                  ];
                case 1:
                  apiOnly = _state.sent();
                  return [
                    4,
                    hookRunners.addRuntimeExports()
                  ];
                case 2:
                  _state.sent();
                  if (!apiOnly)
                    return [
                      3,
                      4
                    ];
                  return [
                    4,
                    hookRunners.modifyServerRoutes({
                      routes: []
                    })
                  ];
                case 3:
                  routes = _state.sent().routes;
                  debug("server routes: %o", routes);
                  appContext = _object_spread_props(_object_spread({}, api.useAppContext()), {
                    apiOnly,
                    serverRoutes: routes
                  });
                  api.setAppContext(appContext);
                  return [
                    2
                  ];
                case 4:
                  return [
                    4,
                    Promise.all([
                      import("./getBundleEntry"),
                      import("./getServerRoutes"),
                      import("./generateCode"),
                      import("./getHtmlTemplate")
                    ])
                  ];
                case 5:
                  _ref = _sliced_to_array.apply(void 0, [
                    _state.sent(),
                    4
                  ]), getBundleEntry = _ref[0].getBundleEntry, getServerRoutes = _ref[1].getServerRoutes, generateCode = _ref[2].generateCode, getHtmlTemplate = _ref[3].getHtmlTemplate;
                  entrypoints = getBundleEntry(appContext, resolvedConfig);
                  debug("entrypoints: %o", entrypoints);
                  initialRoutes = getServerRoutes(entrypoints, {
                    appContext,
                    config: resolvedConfig
                  });
                  return [
                    4,
                    hookRunners.modifyServerRoutes({
                      routes: initialRoutes
                    })
                  ];
                case 6:
                  routes1 = _state.sent().routes;
                  debug("server routes: %o", routes1);
                  appContext = _object_spread_props(_object_spread({}, api.useAppContext()), {
                    entrypoints,
                    serverRoutes: routes1
                  });
                  api.setAppContext(appContext);
                  nestedRouteEntries = entrypoints.map(function(point) {
                    return point.nestedRoutesEntry;
                  }).filter(Boolean);
                  pagesDir = entrypoints.map(function(point) {
                    return point.entry;
                  }).filter(function(entry2) {
                    return entry2 && !path.extname(entry2);
                  }).concat(nestedRouteEntries);
                  originEntrypoints = cloneDeep(entrypoints);
                  return [
                    4,
                    generateCode(appContext, resolvedConfig, entrypoints, api)
                  ];
                case 7:
                  importsStatemets = _state.sent().importsStatemets;
                  return [
                    4,
                    getHtmlTemplate(entrypoints, api, {
                      appContext,
                      config: resolvedConfig
                    })
                  ];
                case 8:
                  htmlTemplates = _state.sent();
                  debug("html templates: %o", htmlTemplates);
                  return [
                    4,
                    hookRunners.addDefineTypes()
                  ];
                case 9:
                  _state.sent();
                  debug("add Define Types");
                  checkedEntries = entrypoints.map(function(point) {
                    return point.entryName;
                  });
                  if (!isDevCommand())
                    return [
                      3,
                      11
                    ];
                  entry = minimist(getArgv()).entry;
                  return [
                    4,
                    getSelectedEntries(typeof entry === "string" ? entry.split(",") : entry, entrypoints)
                  ];
                case 10:
                  checkedEntries = _state.sent();
                  _state.label = 11;
                case 11:
                  appContext = _object_spread_props(_object_spread({}, api.useAppContext()), {
                    entrypoints,
                    checkedEntries,
                    apiOnly,
                    serverRoutes: routes1,
                    htmlTemplates
                  });
                  api.setAppContext(appContext);
                  if (!checkIsBuildCommands())
                    return [
                      3,
                      14
                    ];
                  normalizedConfig = api.useResolvedConfigContext();
                  return [
                    4,
                    createBuilderGenerator(bundler)
                  ];
                case 12:
                  createBuilderForModern = _state.sent();
                  return [
                    4,
                    createBuilderForModern({
                      normalizedConfig,
                      appContext
                    })
                  ];
                case 13:
                  builder = _state.sent();
                  builder.onBeforeBuild(function() {
                    var _ref2 = _async_to_generator(function(param2) {
                      var bundlerConfigs, hookRunners2;
                      return _ts_generator(this, function(_state2) {
                        switch (_state2.label) {
                          case 0:
                            bundlerConfigs = param2.bundlerConfigs;
                            hookRunners2 = api.useHookRunners();
                            return [
                              4,
                              generateRoutes(appContext)
                            ];
                          case 1:
                            _state2.sent();
                            return [
                              4,
                              hookRunners2.beforeBuild({
                                bundlerConfigs
                              })
                            ];
                          case 2:
                            _state2.sent();
                            return [
                              2
                            ];
                        }
                      });
                    });
                    return function(_) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                  builder.onAfterBuild(function() {
                    var _ref2 = _async_to_generator(function(param2) {
                      var stats, hookRunners2;
                      return _ts_generator(this, function(_state2) {
                        switch (_state2.label) {
                          case 0:
                            stats = param2.stats;
                            hookRunners2 = api.useHookRunners();
                            return [
                              4,
                              hookRunners2.afterBuild({
                                stats
                              })
                            ];
                          case 1:
                            _state2.sent();
                            return [
                              4,
                              emitResolvedConfig(appContext.appDirectory, normalizedConfig)
                            ];
                          case 2:
                            _state2.sent();
                            return [
                              2
                            ];
                        }
                      });
                    });
                    return function(_) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                  builder.onDevCompileDone(function() {
                    var _ref2 = _async_to_generator(function(param2) {
                      var isFirstCompile, hookRunners2;
                      return _ts_generator(this, function(_state2) {
                        isFirstCompile = param2.isFirstCompile;
                        hookRunners2 = api.useHookRunners();
                        hookRunners2.afterDev({
                          isFirstCompile
                        });
                        if (isFirstCompile) {
                          printInstructions(hookRunners2, appContext, normalizedConfig);
                        }
                        return [
                          2
                        ];
                      });
                    });
                    return function(_) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                  builder.onBeforeCreateCompiler(function() {
                    var _ref2 = _async_to_generator(function(param2) {
                      var bundlerConfigs, hookRunners2;
                      return _ts_generator(this, function(_state2) {
                        switch (_state2.label) {
                          case 0:
                            bundlerConfigs = param2.bundlerConfigs;
                            hookRunners2 = api.useHookRunners();
                            return [
                              4,
                              generateIndexCode({
                                appContext,
                                config: resolvedConfig,
                                entrypoints,
                                api,
                                importsStatemets,
                                bundlerConfigs
                              })
                            ];
                          case 1:
                            _state2.sent();
                            return [
                              4,
                              hookRunners2.beforeCreateCompiler({
                                bundlerConfigs
                              })
                            ];
                          case 2:
                            _state2.sent();
                            return [
                              2
                            ];
                        }
                      });
                    });
                    return function(_) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                  builder.onAfterCreateCompiler(function() {
                    var _ref2 = _async_to_generator(function(param2) {
                      var compiler, hookRunners2;
                      return _ts_generator(this, function(_state2) {
                        switch (_state2.label) {
                          case 0:
                            compiler = param2.compiler;
                            hookRunners2 = api.useHookRunners();
                            return [
                              4,
                              hookRunners2.afterCreateCompiler({
                                compiler
                              })
                            ];
                          case 1:
                            _state2.sent();
                            return [
                              2
                            ];
                        }
                      });
                    });
                    return function(_) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                  builder.addPlugins(resolvedConfig.builderPlugins);
                  appContext = _object_spread_props(_object_spread({}, api.useAppContext()), {
                    builder
                  });
                  api.setAppContext(appContext);
                  _state.label = 14;
                case 14:
                  return [
                    2
                  ];
              }
            });
          })();
        },
        watchFiles: function watchFiles() {
          return {
            files: pagesDir,
            isPrivate: true
          };
        },
        resolvedConfig: function resolvedConfig(param2) {
          var resolved = param2.resolved;
          var appContext = api.useAppContext();
          var config = initialNormalizedConfig(resolved, appContext, bundler);
          return {
            resolved: config
          };
        },
        modifyEntryImports: (
          // This logic is not in the router plugin to avoid having to include some dependencies in the utils package
          function modifyEntryImports(param2) {
            var entrypoint = param2.entrypoint, imports = param2.imports;
            return _async_to_generator(function() {
              var appContext, srcDirectory, internalSrcAlias, fileSystemRoutes, nestedRoutesEntry, rootLayoutPath, rootLayoutFile, rootLayoutBuffer, rootLayout, _ref, moduleExports, hasAppConfig, generateLayoutPath, hasAppInit;
              return _ts_generator(this, function(_state) {
                switch (_state.label) {
                  case 0:
                    appContext = api.useAppContext();
                    srcDirectory = appContext.srcDirectory, internalSrcAlias = appContext.internalSrcAlias;
                    fileSystemRoutes = entrypoint.fileSystemRoutes, nestedRoutesEntry = entrypoint.nestedRoutesEntry;
                    if (!(fileSystemRoutes && nestedRoutesEntry))
                      return [
                        3,
                        3
                      ];
                    rootLayoutPath = path.join(nestedRoutesEntry, "layout");
                    rootLayoutFile = findExists([
                      ".js",
                      ".ts",
                      ".jsx",
                      ".tsx"
                    ].map(function(ext) {
                      return "".concat(rootLayoutPath).concat(ext);
                    }));
                    if (!rootLayoutFile)
                      return [
                        3,
                        3
                      ];
                    return [
                      4,
                      fs.readFile(rootLayoutFile)
                    ];
                  case 1:
                    rootLayoutBuffer = _state.sent();
                    rootLayout = rootLayoutBuffer.toString();
                    return [
                      4,
                      parseModule({
                        source: rootLayout.toString(),
                        filename: rootLayoutFile
                      })
                    ];
                  case 2:
                    _ref = _sliced_to_array.apply(void 0, [
                      _state.sent(),
                      2
                    ]), moduleExports = _ref[1];
                    hasAppConfig = moduleExports.some(function(e) {
                      return e.n === APP_CONFIG_NAME;
                    });
                    generateLayoutPath = getPathWithoutExt(replaceWithAlias(srcDirectory, rootLayoutFile, internalSrcAlias));
                    if (hasAppConfig) {
                      imports.push({
                        value: generateLayoutPath,
                        specifiers: [
                          {
                            imported: APP_CONFIG_NAME
                          }
                        ]
                      });
                    }
                    hasAppInit = moduleExports.some(function(e) {
                      return e.n === APP_INIT_EXPORTED;
                    });
                    if (hasAppInit) {
                      imports.push({
                        value: generateLayoutPath,
                        specifiers: [
                          {
                            imported: APP_INIT_EXPORTED,
                            local: APP_INIT_IMPORTED
                          }
                        ]
                      });
                    }
                    _state.label = 3;
                  case 3:
                    return [
                      2,
                      {
                        entrypoint,
                        imports
                      }
                    ];
                }
              });
            })();
          }
        ),
        fileChange: function fileChange(e) {
          return _async_to_generator(function() {
            var appContext, appDirectory, filename, eventType, isPageFile, absoluteFilePath, isRouteComponent, resolvedConfig, generateCode, entrypoints;
            return _ts_generator(this, function(_state) {
              switch (_state.label) {
                case 0:
                  appContext = api.useAppContext();
                  appDirectory = appContext.appDirectory;
                  filename = e.filename, eventType = e.eventType;
                  isPageFile = function(name) {
                    return pagesDir.some(function(pageDir) {
                      return name.includes(pageDir);
                    });
                  };
                  absoluteFilePath = path.resolve(appDirectory, filename);
                  isRouteComponent = isPageFile(absoluteFilePath) && isPageComponentFile(absoluteFilePath);
                  if (!(isRouteComponent && (eventType === "add" || eventType === "unlink")))
                    return [
                      3,
                      2
                    ];
                  resolvedConfig = api.useResolvedConfigContext();
                  return [
                    4,
                    import("./generateCode")
                  ];
                case 1:
                  generateCode = _state.sent().generateCode;
                  entrypoints = cloneDeep(originEntrypoints);
                  generateCode(appContext, resolvedConfig, entrypoints, api);
                  _state.label = 2;
                case 2:
                  return [
                    2
                  ];
              }
            });
          })();
        }
      };
    }
  };
}
;
export {
  analyze_default as default
};
