import { _ as _async_to_generator } from "@swc/helpers/_/_async_to_generator";
import { _ as _ts_generator } from "@swc/helpers/_/_ts_generator";
import path from "path";
import os from "node:os";
import { ROUTE_SPEC_FILE, fs as fse, isDepExists } from "@modern-js/utils";
import { parseNodeModulePath } from "mlly";
var getProjectUsage = function(appDirectory, distDirectory) {
  var routeJSON = path.join(distDirectory, ROUTE_SPEC_FILE);
  var routes = fse.readJSONSync(routeJSON).routes;
  var useSSR = false;
  var useAPI = false;
  routes.forEach(function(route) {
    if (route.isSSR) {
      useSSR = true;
    }
    if (route.isApi) {
      useAPI = true;
    }
  });
  var useWebServer = isDepExists(appDirectory, "@modern-js/plugin-server");
  return {
    useSSR,
    useAPI,
    useWebServer
  };
};
function applyProductionCondition(exports) {
  if (!exports || typeof exports === "string") {
    return;
  }
  if (exports.production) {
    if (typeof exports.production === "string") {
      exports.default = exports.production;
    } else {
      Object.assign(exports, exports.production);
    }
  }
  for (var key in exports) {
    applyProductionCondition(exports[key]);
  }
}
function applyPublicCondition(pkg) {
  var _pkg_publishConfig;
  if (pkg === null || pkg === void 0 ? void 0 : (_pkg_publishConfig = pkg.publishConfig) === null || _pkg_publishConfig === void 0 ? void 0 : _pkg_publishConfig.exports) {
    var _pkg_publishConfig1;
    pkg.exports = pkg === null || pkg === void 0 ? void 0 : (_pkg_publishConfig1 = pkg.publishConfig) === null || _pkg_publishConfig1 === void 0 ? void 0 : _pkg_publishConfig1.exports;
  }
}
var writePackage = function() {
  var _ref = _async_to_generator(function(pkg, version, projectDir, _pkgPath) {
    var pkgPath, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, src, subpath, dest, dirname, subpath1, dest1, dirname1, err, pkgJSON, packageJsonPath;
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          pkgPath = _pkgPath || pkg.name;
          _iteratorNormalCompletion = true, _didIteratorError = false, _iteratorError = void 0;
          _state.label = 1;
        case 1:
          _state.trys.push([
            1,
            10,
            11,
            12
          ]);
          _iterator = pkg.versions[version].files[Symbol.iterator]();
          _state.label = 2;
        case 2:
          if (!!(_iteratorNormalCompletion = (_step = _iterator.next()).done))
            return [
              3,
              9
            ];
          src = _step.value;
          if (!src.includes("node_modules"))
            return [
              3,
              5
            ];
          subpath = parseNodeModulePath(src).subpath;
          dest = path.join(projectDir, "node_modules", pkgPath, subpath);
          dirname = path.dirname(dest);
          return [
            4,
            fse.ensureDir(dirname)
          ];
        case 3:
          _state.sent();
          return [
            4,
            fse.copyFile(src, dest)
          ];
        case 4:
          _state.sent();
          return [
            3,
            8
          ];
        case 5:
          subpath1 = path.relative(pkg.versions[version].path, src);
          dest1 = path.join(projectDir, "node_modules", pkgPath, subpath1);
          dirname1 = path.dirname(dest1);
          return [
            4,
            fse.ensureDir(dirname1)
          ];
        case 6:
          _state.sent();
          return [
            4,
            fse.copyFile(src, dest1)
          ];
        case 7:
          _state.sent();
          _state.label = 8;
        case 8:
          _iteratorNormalCompletion = true;
          return [
            3,
            2
          ];
        case 9:
          return [
            3,
            12
          ];
        case 10:
          err = _state.sent();
          _didIteratorError = true;
          _iteratorError = err;
          return [
            3,
            12
          ];
        case 11:
          try {
            if (!_iteratorNormalCompletion && _iterator.return != null) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
          return [
            7
          ];
        case 12:
          pkgJSON = pkg.versions[version].pkgJSON;
          applyPublicCondition(pkgJSON);
          packageJsonPath = path.join(projectDir, "node_modules", pkgPath, "package.json");
          return [
            4,
            fse.ensureDir(path.dirname(packageJsonPath))
          ];
        case 13:
          _state.sent();
          return [
            4,
            fse.writeFile(packageJsonPath, JSON.stringify(pkgJSON, null, 2))
          ];
        case 14:
          _state.sent();
          return [
            2
          ];
      }
    });
  });
  return function writePackage2(pkg, version, projectDir, _pkgPath) {
    return _ref.apply(this, arguments);
  };
}();
var isWindows = os.platform() === "win32";
var linkPackage = function() {
  var _ref = _async_to_generator(function(from, to, projectRootDir) {
    var src, dest, dstStat, exists;
    return _ts_generator(this, function(_state) {
      switch (_state.label) {
        case 0:
          src = path.join(projectRootDir, "node_modules", from);
          dest = path.join(projectRootDir, "node_modules", to);
          return [
            4,
            fse.lstat(dest).catch(function() {
              return null;
            })
          ];
        case 1:
          dstStat = _state.sent();
          exists = dstStat === null || dstStat === void 0 ? void 0 : dstStat.isSymbolicLink();
          if (exists) {
            return [
              2
            ];
          }
          return [
            4,
            fse.mkdir(path.dirname(dest), {
              recursive: true
            })
          ];
        case 2:
          _state.sent();
          return [
            4,
            fse.symlink(path.relative(path.dirname(dest), src), dest, isWindows ? "junction" : "dir").catch(function(error) {
              console.error("Cannot link", from, "to", to, error);
            })
          ];
        case 3:
          _state.sent();
          return [
            2
          ];
      }
    });
  });
  return function linkPackage2(from, to, projectRootDir) {
    return _ref.apply(this, arguments);
  };
}();
export {
  applyProductionCondition,
  applyPublicCondition,
  getProjectUsage,
  linkPackage,
  writePackage
};
